package App::Luser;

use Moose;
with 'MooseX::Getopt::Dashes';
with 'MooseX::SimpleConfig';
use utf8;

use Text::Password::Pronounceable::RandomCase;
use IO::Prompter;
use Term::ReadKey;
use Net::LDAP;
use Net::LDAP::Extension::SetPassword;
use List::Util qw(max);
use Config::Find;
use Email::Sender::Simple qw(sendmail);
use Email::Simple;
use Email::Simple::Creator;

our $VERSION = '0.26';

has user      => ( is => 'rw', isa => 'Str', required => 1 );
has manager   => ( is => 'rw', isa => 'Str' );
has sn        => ( is => 'rw', isa => 'Str', required => 1 );
has givenname => ( is => 'rw', isa => 'Str', required => 1 );
has cn        => ( is => 'rw', isa => 'Str' );
has title     => ( is => 'rw', isa => 'Str' );
has gecos     => ( is => 'rw', isa => 'Str' );
has mailto    => ( is => 'rw', isa => 'Str', default => 'support@math.uni-bonn.de' );
has room      => ( is => 'rw', isa => 'Str' );
has phone     => ( is => 'rw', isa => 'Str' );
has type      => ( is => 'rw', isa => 'Str', required => 1 );
has host      => ( is => 'rw', isa => 'Str', default  => 'ldap' );
has user_password => ( is => 'rw', isa => 'Str' );
has ldap_password => ( is => 'rw', isa => 'Str' );
has generate_password => ( is => 'rw', isa => 'Bool', default => 1 );
has omit_room => ( is => 'rw', isa => 'Bool' );
has omit_phone => ( is => 'rw', isa => 'Bool' );
has omit_manager => ( is => 'rw', isa => 'Bool' );
has omit_mail => ( is => 'rw', isa => 'Bool' );
has omit_gecos => ( is => 'rw', isa => 'Bool' );
has fileserver => ( is => 'rw', isa => 'Str', default => 'rhein' );

has configfile => (
    is      => 'rw',
    isa     => 'Maybe[Str]',
    default => Config::Find->find( name => 'luser.yaml' )
);

has uidnumber => (
    is      => 'rw',
    isa     => 'Str',
    lazy    => 1,
    default => sub { shift->next_uid() },
);

has ldap => (
    is      => 'rw',
    isa     => 'Net::LDAP',
    lazy    => 1,
    default => sub {
        my $self = shift;
        my $ldap = Net::LDAP->new( $self->host() )
          or die "Can't connect to ldap server: $!\n";
        my $mesg =
          $ldap->bind( $self->bind_as(), password => $self->ldap_password() );
        $mesg->code() && die $mesg->error . "\n";
        return $ldap;
    },
);

has bind_as => (
    is      => 'rw',
    isa     => 'Str',
    lazy    => 1,
    default => sub { "uid=$ENV{USER}," . shift->user_base }
);

has base => (
    is      => 'rw',
    isa     => 'Str',
    default => "dc=math,dc=uni-bonn,dc=de"
);
has user_base => ( is => 'rw', isa => 'Str' );
has map_base => ( is => 'rw', isa => 'Str' );

has filter => (
    is      => 'rw',
    isa     => 'Str',
    lazy    => 1,
    default => sub { return "(&(employeeType=*)(uid=" . shift->user . "))" },
);

has dn => (
    is      => 'ro',
    isa     => 'Str',
    lazy    => 1,
    default => sub { return "uid=" . $_[0]->user . "," . $_[0]->user_base },
);


sub BUILD {
    my ($self) = @_;

    $self->user_base( "ou=people," . $self->base );
    $self->map_base( "ou=amd maps," . $self->base );

    if ( $self->generate_password() ) {
        $self->user_password(
            Text::Password::Pronounceable::RandomCase->generate( 8, 12 ) );
    }

    else {
        $self->user_password( "" . prompt( 'Password (User): ', -stdio, -echo => '*' ) );
    }

    if ( !$self->ldap_password() ) {
        $self->ldap_password( "" . prompt( 'Password (LDAP): ', -stdio, -echo => '*' ) );
    }

    if ( $self->generate_password and !$self->mailto ) {
        die "Mailto required when autogenerating passwords\n";
    }
    die "Missing room (and no omit-room)\n" unless $self->room or $self->omit_room;
    die "Missing phone (and no omit-phone)\n" unless $self->phone or $self->omit_phone;
    die "Missing manager (and no omit-manager)\n" unless $self->manager or $self->omit_manager;
}

sub next_uid {
    my ($self) = @_;
    my $mesg = $self->ldap->search(
        base   => $self->base(),
        filter => 'employeeType=*',
        attrs  => ['uidNumber'],
    );
    $mesg->code() && die $mesg->error . "\n";
    if ( $mesg->count() ) {
        return 1 +  max map { $_->get_value('uidNumber') } $mesg->entries;
    }
    else {
        die "No uids found.\n";
    }
}

sub is_ascii {
	my $s = shift;
	return $s =~ m/^\p{IsASCII}*$/
}

sub to_ascii {
	my $s = shift;
	$s =~ s/(.)/{
		'Ä' => 'Ae',
		'Ö' => 'Oe',
		'Ü' => 'Ue',
		'ä' => 'ae',
		'ö' => 'oe',
		'ü' => 'ue',
		'ß' => 'ss',
	}->{$1}||$1/ge;
	die "Can't fully convert to ASCII: $s\n" unless is_ascii($s);
	return $s;
}

sub create_user {
    my ($self) = @_;

    my $user  = $self->user();
    die "Username must be ASCII\n" unless is_ascii($user);
    die "Uppercase characters in username\n" if $user =~ m/[A-Z]/;
    die "Illegal characters in username\n" unless $user =~ m/^[a-z0-9.-_]*$/;
    die "Username too long\n" if length($user) > 8;
    my $entry = Net::LDAP::Entry->new;
    $entry->dn( $self->dn() );
    my $full_name = $self->givenname . " " . $self->sn;
    my $cn = $self->cn || $full_name;
    my $gecos = $self->gecos || to_ascii($full_name);
    die "GECOS must be ASCII\n" unless is_ascii($gecos);
    my %attrs = (
        objectClass => [
            qw( top person organizationalPerson inetOrgPerson posixAccount shadowAccount)
        ],
        uid             => "$user",
        loginShell      => "/bin/bash",
        givenName       => $self->givenname,
        sn              => $self->sn,
        cn              => $cn,
        homeDirectory   => "/home/$user",
        employeeType    => $self->type(),
        gidNumber       => 100,
        uidNumber       => $self->uidnumber(),
    );
    $attrs{mail}            = "$user\@math.uni-bonn.de" unless $self->omit_mail;
    $attrs{gecos}           = $gecos unless $self->omit_gecos;
    $attrs{manager}         = "uid=" . $self->manager . "," . $self->user_base() unless $self->omit_manager;
    $attrs{roomNumber}      = $self->room() unless $self->omit_room;
    $attrs{telephoneNumber} = $self->phone() unless $self->omit_phone;
    $attrs{title}           = $self->title() if $self->title();
    # Net::LDAP gibt sonst evtl. Latin-1 heraus, obwohl das UTF-8 sein muß:
    utf8::upgrade($attrs{givenName}); utf8::upgrade($attrs{sn}); utf8::upgrade($attrs{cn});
    $entry->add(%attrs);
    return $entry;
}

sub create_map {
    my ($self) = @_;
    my $user   = $self->user();
    my $entry  = Net::LDAP::Entry->new;
    my $fileserver = $self->fileserver;
    $entry->dn( "cn=amdmap home[$user]," . $self->map_base );
    $entry->add(
        objectClass => [ qw( amdmap ) ],
        cn          => "amdmap home[$user]",
        amdmapName  => 'home',
        amdmapKey   => $user,
        amdmapValue => "rhost:=$fileserver",
    );
    return $entry;
}

sub run {
    my ($self) = @_;

    my $ldap = $self->ldap();
    my $mesg = $ldap->search(    # perform a search
        base   => $self->base,
        filter => $self->filter,
    );

    $mesg->code() && die $mesg->error . "\n";

    if ( $mesg->count() ) {
        die $self->user . " already exists.\n";
    }
    else {
        my $entry = $self->create_user();
        $mesg = $entry->update($ldap);
        $mesg->code() && die $mesg->error . "\n";

        my $map = $self->create_map();
        $mesg = $map->update($ldap);
        $mesg->code() && die $mesg->error . "\n";

        $mesg = $ldap->set_password(
            user      => $self->dn(),
            newpasswd => $self->user_password
        );
        $mesg->code() && die $mesg->error . "\n";

	## Okay, Templating fuer arme
	my $body = do { $/ = undef; <DATA> };
	$body =~ s/USERNAME/$self->user/eg;
	$body =~ s/PASSWORD/$self->user_password/eg;

        my $email = Email::Simple->create(
            header => [
                To      => $self->mailto,
                From    => 'support@math.uni-bonn.de',
                Subject => "New account for " . $self->user,
            ],
            body => $body
        );
        sendmail($email);
    }

}

=head1 NAME

App::Luser - The great new App::Luser!

=head1 VERSION

Version 0.08


=head1 SYNOPSIS

Quick summary of what the module does.

Perhaps a little code snippet.

    use App::Luser;

    my $foo = App::Luser->new();
    ...

=head1 EXPORT

A list of functions that can be exported.  You can delete this section
if you don't export anything, such as for a purely object-oriented module.

=head1 FUNCTIONS

=head2 function1

=cut

sub function1 {
}

=head2 function2

=cut

sub function2 {
}

=head1 AUTHOR

Mario Domgoergen, C<< <dom at math.uni-bonn.de> >>

=head1 BUGS

Please report any bugs or feature requests to C<bug-app-luser at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=App-Luser>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.


=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc App::Luser


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=App-Luser>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/App-Luser>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/App-Luser>

=item * Search CPAN

L<http://search.cpan.org/dist/App-Luser>

=back

=head1 ACKNOWLEDGEMENTS

=head1 COPYRIGHT & LICENSE

Copyright 2009 Mario Domgoergen, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1;    # End of App::Luser

__DATA__
Guten Tag.

Wir haben soeben auf Ihre Anfrage hin ein neues Benutzerkonto eingerichtet.
Der Benutzer kann sich unter dem Namen ,,USERNAME'' mit dem Kennwort ,,PASSWORD'' auf den Institutsrechnern anmelden.
Bitte beachten Sie, daß dieses MI-Benutzerkonto unabhängig und verschieden von einem möglicherweise ebenfalls eingerichteten zentralen Uni-Benutzerkonto (,,Uni-Id'', enthält kein ,,.math'') ist.

Das MI-Konto beinhaltet die MI-Mailadresse <USERNAME@math.uni-bonn.de>. Da Ankündigungen des Institues und der EDV an diese Adresse verschickt werden, wird von allen Benutzern erwartet, daß sie dort eintreffende Mails lesen.

Sollten irgendwelche Probleme auftreten, melden Sie sich bitte bei uns.
Für eine Übersicht unseres Serviceangebotes besuchen Sie https://www.math.uni-bonn.de/people/support/informationen/
Falls Sie das MI-WLAN benutzen wollen, fordern Sie bitte bei uns ein Zugangszertifikat an und folgen Sie den dort angegebenen Anleitungen.

Viele Grüße, 
Das Supportteam 



Dear applicant,

we just generated a new account on your request.  The user can log in as user ``USERNAME'' with password ``PASSWORD''.
Please note that this MI account is independent and distinct from any central account (``Uni Id'', no ``.math'' in it) that may have been provided to you.

The MI account includes the email address <USERNAME@math.uni-bonn.de>.  As institute and IT announcements will be sent to this address, all users are expected to read mails addressed to it.

If there are any problems, please let us know.
For an overview of the IT services provided by us please visit https://www.math.uni-bonn.de/people/support/informationen/?language=en
In order to use the MI WiFi, please request a certificate with us and follow the instructions given there.

Best regards,
the support
